import { useContext, useRef, useMemo, useCallback } from 'react';
import '../utils/useSealedState.js';
import '../utils/useId.js';
import { unstable_useCreateElement } from '../utils/useCreateElement.js';
import '../system/SystemContext.js';
import '../chunk-988837cf.js';
import '../chunk-56719312.js';
import { unstable_mergeProps } from '../utils/mergeProps.js';
import { a as _objectSpread } from '../chunk-adf96f9e.js';
import '../system/useToken.js';
import '../system/useOptions.js';
import '../system/useProps.js';
import '../chunk-2f16e450.js';
import { unstable_createHook } from '../utils/createHook.js';
import { unstable_createComponent } from '../utils/createComponent.js';
import '../Box/Box.js';
import '../chunk-6e18ef4b.js';
import '../Tabbable/Tabbable.js';
import '../chunk-29332083.js';
import { a as warning } from '../chunk-e555b6ea.js';
import 'react-dom';
import '../Portal/Portal.js';
import '../Hidden/HiddenState.js';
import '../Hidden/Hidden.js';
import '../Dialog/Dialog.js';
import 'body-scroll-lock';
import '../chunk-b4d83ecc.js';
import '../Dialog/DialogState.js';
import { a as createOnKeyDown } from '../chunk-96f01ae3.js';
import '../Rover/RoverState.js';
import '../Rover/Rover.js';
import 'popper.js';
import '../Popover/PopoverState.js';
import { usePopover } from '../Popover/Popover.js';
import { useStaticMenu } from './StaticMenu.js';
import { a as MenuContext } from '../chunk-ee62c270.js';
import { useMenuState } from './MenuState.js';

var useMenu = unstable_createHook({
  name: "Menu",
  compose: [useStaticMenu, usePopover],
  useState: useMenuState,
  propsAreEqual: function propsAreEqual(prev, next) {
    if (prev.visible === false && next.visible === false) {
      return true;
    }

    return null;
  },
  useOptions: function useOptions(options) {
    var parent = useContext(MenuContext);
    var parentIsHorizontal = parent && parent.orientation === "horizontal";
    return _objectSpread({
      unstable_autoFocusOnShow: !parent,
      unstable_autoFocusOnHide: !parentIsHorizontal
    }, options);
  },
  useProps: function useProps(options, htmlProps) {
    var parent = useContext(MenuContext);
    var ref = useRef(null);
    var isHorizontal = options.orientation === "horizontal";
    var isVertical = options.orientation === "vertical";
    var horizontalParent = parent;

    while (horizontalParent && horizontalParent.orientation !== "horizontal") {
      horizontalParent = horizontalParent.parent;
    }

    var _options$placement$sp = options.placement.split("-"),
        dir = _options$placement$sp[0];

    var rovingBindings = useMemo(function () {
      return createOnKeyDown({
        stopPropagation: function stopPropagation(event) {
          // On Esc, only stop propagation if there's no parent menu
          // Otherwise, pressing Esc should close all menus
          if (event.key === "Escape" && parent) return false;
          return true;
        },
        keyMap: function keyMap(event) {
          process.env.NODE_ENV !== "production" ? warning(!ref.current, "Can't detect arrow keys because `ref` wasn't passed to component. See https://reakit.io/docs/menu", "Menu") : void 0;
          var targetIsMenu = event.target === ref.current;
          return {
            Escape: options.hide,
            ArrowUp: targetIsMenu && !isHorizontal && options.last,
            ArrowRight: targetIsMenu && !isVertical && options.first,
            ArrowDown: targetIsMenu && !isHorizontal && options.first,
            ArrowLeft: targetIsMenu && !isVertical && options.last,
            Home: targetIsMenu && options.first,
            End: targetIsMenu && options.last,
            PageUp: targetIsMenu && options.first,
            PageDown: targetIsMenu && options.last
          };
        }
      });
    }, [parent, options.hide, options.last, options.first]);
    var parentBindings = useMemo(function () {
      return createOnKeyDown({
        stopPropagation: true,
        shouldKeyDown: function shouldKeyDown(event) {
          return Boolean(parent && ref.current && ref.current.contains(event.target));
        },
        keyMap: parent ? {
          ArrowRight: horizontalParent && dir !== "left" ? horizontalParent.next : dir === "left" && options.hide,
          ArrowLeft: horizontalParent && dir !== "right" ? horizontalParent.previous : dir === "right" && options.hide
        } : {}
      });
    }, [Boolean(parent), horizontalParent && horizontalParent.next, horizontalParent && horizontalParent.previous, dir, options.hide]);
    var onKeyDown = useCallback(function (event) {
      rovingBindings(event);
      parentBindings(event);
    }, [rovingBindings, parentBindings]);
    return unstable_mergeProps({
      ref: ref,
      role: "menu",
      onKeyDown: onKeyDown
    }, htmlProps);
  },
  useCompose: function useCompose(options, htmlProps) {
    htmlProps = useStaticMenu(options, htmlProps);
    return usePopover(_objectSpread({}, options, {
      modal: false,
      unstable_portal: false,
      unstable_orphan: false,
      hideOnEsc: false
    }), htmlProps);
  }
});
var Menu = unstable_createComponent({
  as: "div",
  useHook: useMenu,
  useCreateElement: function useCreateElement(type, props, children) {
    process.env.NODE_ENV !== "production" ? warning(!props["aria-label"] && !props["aria-labelledby"], "You should provide either `aria-label` or `aria-labelledby` props. See https://reakit.io/docs/menu", "Menu") : void 0;
    return unstable_useCreateElement(type, props, children);
  }
});

export { Menu, useMenu };
