import { useContext, useRef, useState, useEffect, useMemo, useCallback } from 'react';
import '../utils/useSealedState.js';
import '../utils/useId.js';
import '../utils/useCreateElement.js';
import '../system/SystemContext.js';
import '../chunk-988837cf.js';
import '../chunk-56719312.js';
import { unstable_mergeProps } from '../utils/mergeProps.js';
import '../system/SystemProvider.js';
import '../utils/Provider.js';
import { a as _objectSpread } from '../chunk-adf96f9e.js';
import '../system/useToken.js';
import '../system/useOptions.js';
import '../system/useProps.js';
import '../chunk-2f16e450.js';
import { unstable_createHook } from '../utils/createHook.js';
import { unstable_createComponent } from '../utils/createComponent.js';
import '../Box/Box.js';
import '../chunk-6e18ef4b.js';
import '../Tabbable/Tabbable.js';
import '../Button/Button.js';
import { a as warning } from '../chunk-e555b6ea.js';
import '../Hidden/HiddenState.js';
import '../chunk-b4d83ecc.js';
import '../Dialog/DialogState.js';
import '../Hidden/HiddenDisclosure.js';
import '../Dialog/DialogDisclosure.js';
import { a as createOnKeyDown } from '../chunk-96f01ae3.js';
import '../Rover/RoverState.js';
import '../Rover/Rover.js';
import 'popper.js';
import '../Popover/PopoverState.js';
import { a as MenuContext } from '../chunk-ee62c270.js';
import { useMenuState } from './MenuState.js';
import { usePopoverDisclosure } from '../Popover/PopoverDisclosure.js';

var noop = function noop() {};

var useMenuDisclosure = unstable_createHook({
  name: "MenuDisclosure",
  compose: usePopoverDisclosure,
  useState: useMenuState,
  useProps: function useProps(options, htmlProps) {
    var parent = useContext(MenuContext);
    var ref = useRef(null); // This avoids race condition between focus and click.
    // On some browsers, focus is triggered right before click.
    // So we use it to disable toggling.

    var _React$useState = useState(false),
        hasShownOnFocus = _React$useState[0],
        setHasShownOnFocus = _React$useState[1];

    var _options$placement$sp = options.placement.split("-"),
        dir = _options$placement$sp[0]; // Restores hasShownOnFocus


    useEffect(function () {
      if (hasShownOnFocus) {
        setTimeout(function () {
          return setHasShownOnFocus(false);
        }, 200);
      }
    }, [hasShownOnFocus]);
    var onKeyDown = useMemo(function () {
      return createOnKeyDown({
        stopPropagation: function stopPropagation(event) {
          return event.key !== "Escape";
        },
        onKey: options.show,
        keyMap: function keyMap() {
          var first = function first() {
            return setTimeout(options.first);
          };

          return {
            Escape: options.hide,
            Enter: parent && first,
            " ": parent && first,
            ArrowUp: dir === "top" || dir === "bottom" ? options.last : false,
            ArrowRight: dir === "right" && first,
            ArrowDown: dir === "bottom" || dir === "top" ? first : false,
            ArrowLeft: dir === "left" && first
          };
        }
      });
    }, [options.show, options.hide, options.first, options.last]);
    var onFocus = useCallback(function () {
      if (parent && parent.orientation === "horizontal") {
        setHasShownOnFocus(true);
        options.show();
      }
    }, [parent && parent.orientation, setHasShownOnFocus, options.show]);
    var onMouseOver = useCallback(function () {
      if (!parent) return;

      if (!ref.current) {
        process.env.NODE_ENV !== "production" ? warning(true, "Can't respond to mouse over on `MenuDisclosure` because `ref` wasn't passed to component. See https://reakit.io/docs/menu", "MenuDisclosure") : void 0;
        return;
      }

      var parentIsHorizontal = parent.orientation === "horizontal";

      if (!parentIsHorizontal) {
        setTimeout(function () {
          if (ref.current && ref.current.contains(document.activeElement)) {
            options.show();
            ref.current.focus();
          }
        }, 200);
      } else {
        var parentMenu = ref.current.closest("[role=menu],[role=menubar]");
        var subjacentOpenMenu = parentMenu && parentMenu.querySelector("[role=menu]:not([hidden])");

        if (subjacentOpenMenu) {
          ref.current.focus();
        }
      }
    }, [parent && parent.orientation, options.show]);
    return unstable_mergeProps({
      ref: ref,
      "aria-haspopup": "menu",
      onClick: parent && (parent.orientation !== "horizontal" || hasShownOnFocus) ? options.show : options.toggle,
      onFocus: onFocus,
      onMouseOver: onMouseOver,
      onKeyDown: onKeyDown
    }, htmlProps);
  },
  useCompose: function useCompose(options, htmlProps) {
    // Toggling is handled by MenuDisclosure
    return usePopoverDisclosure(_objectSpread({}, options, {
      toggle: noop
    }), htmlProps);
  }
});
var MenuDisclosure = unstable_createComponent({
  as: "button",
  useHook: useMenuDisclosure
});

export { MenuDisclosure, useMenuDisclosure };
