import { useRef, useEffect, useCallback, useMemo } from 'react';
import '../utils/useSealedState.js';
import { unstable_useId } from '../utils/useId.js';
import '../utils/useCreateElement.js';
import '../system/SystemContext.js';
import '../chunk-988837cf.js';
import '../chunk-56719312.js';
import { unstable_mergeProps } from '../utils/mergeProps.js';
import { b as _objectWithoutPropertiesLoose } from '../chunk-adf96f9e.js';
import '../system/useToken.js';
import '../system/useOptions.js';
import '../system/useProps.js';
import '../chunk-2f16e450.js';
import { unstable_createHook } from '../utils/createHook.js';
import { unstable_createComponent } from '../utils/createComponent.js';
import '../Box/Box.js';
import '../chunk-6e18ef4b.js';
import { useTabbable } from '../Tabbable/Tabbable.js';
import { a as warning } from '../chunk-e555b6ea.js';
import { a as useUpdateEffect } from '../chunk-b4d83ecc.js';
import { a as createOnKeyDown } from '../chunk-96f01ae3.js';
import { useRoverState } from './RoverState.js';

var useRover = unstable_createHook({
  name: "Rover",
  compose: useTabbable,
  useState: useRoverState,
  keys: ["stopId"],
  useProps: function useProps(options, _ref) {
    var tabIndex = _ref.tabIndex,
        htmlOnKeyDown = _ref.onKeyDown,
        htmlProps = _objectWithoutPropertiesLoose(_ref, ["tabIndex", "onKeyDown"]);

    var ref = useRef(null);
    var id = unstable_useId("rover-");
    var stopId = options.stopId || htmlProps.id || id;
    var trulyDisabled = options.disabled && !options.focusable;
    var noFocused = options.currentId == null;
    var focused = options.currentId === stopId;
    var isFirst = options.stops[0] && options.stops[0].id === stopId;
    var shouldTabIndex = focused || isFirst && noFocused;
    useEffect(function () {
      if (trulyDisabled) return undefined;
      options.register(stopId, ref);
      return function () {
        return options.unregister(stopId);
      };
    }, [stopId, trulyDisabled, options.register, options.unregister]);
    useUpdateEffect(function () {
      if (!ref.current) {
        process.env.NODE_ENV !== "production" ? warning(true, "Can't focus rover component because either `ref` wasn't passed to component or the component wasn't rendered. See https://reakit.io/docs/rover", "Rover") : void 0;
        return;
      }

      if (focused) {
        ref.current.focus();
      }
    }, [focused]);
    var onFocus = useCallback(function () {
      return options.move(stopId);
    }, [options.move, stopId]);
    var onKeyDown = useMemo(function () {
      return createOnKeyDown({
        onKeyDown: htmlOnKeyDown,
        keyMap: {
          ArrowUp: options.orientation !== "horizontal" && options.previous,
          ArrowRight: options.orientation !== "vertical" && options.next,
          ArrowDown: options.orientation !== "horizontal" && options.next,
          ArrowLeft: options.orientation !== "vertical" && options.previous,
          Home: options.first,
          End: options.last,
          PageUp: options.first,
          PageDown: options.last
        }
      });
    }, [htmlOnKeyDown, options.orientation, options.previous, options.next, options.first, options.last]);
    return unstable_mergeProps({
      ref: ref,
      id: stopId,
      tabIndex: shouldTabIndex ? tabIndex : -1,
      onFocus: onFocus,
      onKeyDown: onKeyDown
    }, htmlProps);
  }
});
var Rover = unstable_createComponent({
  as: "button",
  useHook: useRover
});

export { Rover, useRover };
