import { useRef, useCallback } from 'react';
import '../utils/useCreateElement.js';
import '../system/SystemContext.js';
import '../chunk-988837cf.js';
import '../chunk-56719312.js';
import { unstable_mergeProps } from '../utils/mergeProps.js';
import { b as _objectWithoutPropertiesLoose, a as _objectSpread } from '../chunk-adf96f9e.js';
import '../system/useToken.js';
import '../system/useOptions.js';
import '../system/useProps.js';
import '../chunk-2f16e450.js';
import { unstable_createHook } from '../utils/createHook.js';
import { unstable_createComponent } from '../utils/createComponent.js';
import { useBox } from '../Box/Box.js';
import { a as useLiveRef } from '../chunk-6e18ef4b.js';

function isNativeTabbable(element) {
  if (element instanceof HTMLButtonElement) return true;
  if (element instanceof HTMLInputElement) return true;
  if (element instanceof HTMLSelectElement) return true;
  if (element instanceof HTMLTextAreaElement) return true;
  if (element instanceof HTMLAnchorElement) return true;
  if (element instanceof HTMLAudioElement) return true;
  if (element instanceof HTMLVideoElement) return true;
  return false;
}

var defaultClickKeys = ["Enter", " "];
var useTabbable = unstable_createHook({
  name: "Tabbable",
  compose: useBox,
  keys: ["disabled", "focusable", "unstable_clickKeys"],
  useOptions: function useOptions(_ref, htmlProps) {
    var _ref$unstable_clickKe = _ref.unstable_clickKeys,
        unstable_clickKeys = _ref$unstable_clickKe === void 0 ? defaultClickKeys : _ref$unstable_clickKe,
        options = _objectWithoutPropertiesLoose(_ref, ["unstable_clickKeys"]);

    return _objectSpread({
      unstable_clickKeys: unstable_clickKeys,
      disabled: htmlProps.disabled
    }, options);
  },
  useProps: function useProps(options, _ref2) {
    var _ref2$tabIndex = _ref2.tabIndex,
        tabIndex = _ref2$tabIndex === void 0 ? 0 : _ref2$tabIndex,
        onClick = _ref2.onClick,
        onMouseOver = _ref2.onMouseOver,
        onMouseDown = _ref2.onMouseDown,
        htmlProps = _objectWithoutPropertiesLoose(_ref2, ["tabIndex", "onClick", "onMouseOver", "onMouseDown"]);

    var ref = useRef(null);
    var clickKeysRef = useLiveRef(options.unstable_clickKeys);
    var trulyDisabled = options.disabled && !options.focusable;
    return unstable_mergeProps({
      ref: ref,
      disabled: trulyDisabled,
      tabIndex: trulyDisabled ? undefined : tabIndex,
      "aria-disabled": options.disabled,
      onMouseDown: useCallback(function (event) {
        if (event.target instanceof HTMLInputElement) {
          if (onMouseDown) {
            onMouseDown(event);
          }

          return;
        }

        event.preventDefault();

        if (options.disabled) {
          event.stopPropagation();
        } else {
          (ref.current || event.target).focus();

          if (onMouseDown) {
            onMouseDown(event);
          }
        }
      }, [options.disabled, onMouseDown]),
      onClick: useCallback(function (event) {
        if (options.disabled) {
          event.stopPropagation();
          event.preventDefault();
        } else if (onClick) {
          onClick(event);
        }
      }, [options.disabled, onClick]),
      onMouseOver: useCallback(function (event) {
        if (options.disabled) {
          event.stopPropagation();
          event.preventDefault();
        } else if (onMouseOver) {
          onMouseOver(event);
        }
      }, [options.disabled, onMouseOver]),
      onKeyDown: useCallback(function (event) {
        if (options.disabled) return;

        if (clickKeysRef.current === defaultClickKeys && isNativeTabbable(event.target)) {
          return;
        }

        if (clickKeysRef.current && clickKeysRef.current.indexOf(event.key) !== -1) {
          event.preventDefault();
          event.target.dispatchEvent(new MouseEvent("click", {
            view: window,
            bubbles: true,
            cancelable: false
          }));
        }
      }, [options.disabled])
    }, htmlProps);
  }
});
var Tabbable = unstable_createComponent({
  as: "button",
  useHook: useTabbable
});

export { Tabbable, useTabbable };
