import { a as reduceObjects } from '../chunk-988837cf.js';
import { a as isObject } from '../chunk-56719312.js';

function extractPropFromObjects(objects, prop) {
  var props = [];

  for (var _iterator = objects, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
    var _ref;

    if (_isArray) {
      if (_i >= _iterator.length) break;
      _ref = _iterator[_i++];
    } else {
      _i = _iterator.next();
      if (_i.done) break;
      _ref = _i.value;
    }

    var object = _ref;
    var p = object[prop];
    if (p) props.push(p);
  }

  return props;
}

function mergeFunctionsInObjects(objects) {
  var object = reduceObjects(objects, function (value) {
    return typeof value === "function";
  });
  var keys = Object.keys(object);
  var result = {};

  for (var _i2 = 0, _keys = keys; _i2 < _keys.length; _i2++) {
    var key = _keys[_i2];
    var fns = object[key];
    result[key] = fns.length === 1 ? fns[0] : fns.reduce(function (lastFn, currFn) {
      return function () {
        lastFn.apply(void 0, arguments);
        return currFn.apply(void 0, arguments);
      };
    });
  }

  return result;
}

function mergeWrapInObjects(objects) {
  var wrappers = extractPropFromObjects(objects, "unstable_wrap");
  var length = wrappers.length;
  if (length === 0) return {};
  if (length === 1) return {
    unstable_wrap: wrappers[0]
  };
  return {
    unstable_wrap: wrappers.reduce(function (acc, curr) {
      return function (children) {
        return curr(acc(children));
      };
    })
  };
}

function mergeRefsInObjects(objects) {
  var refs = extractPropFromObjects(objects, "ref");
  var length = refs.length;
  if (length === 0) return {};
  if (length === 1) return {
    ref: refs[0]
  };
  return {
    ref: function ref(instance) {
      for (var _iterator2 = refs, _isArray2 = Array.isArray(_iterator2), _i3 = 0, _iterator2 = _isArray2 ? _iterator2 : _iterator2[Symbol.iterator]();;) {
        var _ref2;

        if (_isArray2) {
          if (_i3 >= _iterator2.length) break;
          _ref2 = _iterator2[_i3++];
        } else {
          _i3 = _iterator2.next();
          if (_i3.done) break;
          _ref2 = _i3.value;
        }

        var ref = _ref2;

        if (typeof ref === "function") {
          ref(instance);
        } else if (ref) {
          ref.current = instance;
        }
      }
    }
  };
}

function mergeClassNamesInObjects(objects) {
  var classNames = extractPropFromObjects(objects, "className");
  if (classNames.length === 0) return {};
  return {
    className: classNames.join(" ")
  };
}

function mergeStylesInObjects(objects) {
  var styles = extractPropFromObjects(objects, "style");
  if (styles.length === 0) return {};
  if (styles.length === 1) return {
    style: styles[0]
  };
  return {
    style: Object.assign.apply(Object, [{}].concat(styles))
  };
}

function unstable_mergeProps() {
  for (var _len = arguments.length, objects = new Array(_len), _key = 0; _key < _len; _key++) {
    objects[_key] = arguments[_key];
  }

  objects = objects.filter(isObject);
  return Object.assign.apply(Object, [{}].concat(objects, [mergeFunctionsInObjects(objects), mergeWrapInObjects(objects), mergeRefsInObjects(objects), mergeClassNamesInObjects(objects), mergeStylesInObjects(objects)]));
}

export { unstable_mergeProps };
