'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
require('../utils/useSealedState.js');
require('../utils/useCreateElement.js');
require('../system/SystemContext.js');
require('../chunk-8e8e1bad.js');
require('../chunk-2138b837.js');
var mergeProps = require('../utils/mergeProps.js');
var __chunk_3 = require('../chunk-37dea005.js');
require('../system/useToken.js');
require('../system/useOptions.js');
require('../system/useProps.js');
require('../chunk-c06eec31.js');
var createHook = require('../utils/createHook.js');
var createComponent = require('../utils/createComponent.js');
require('../Box/Box.js');
require('../chunk-9263130a.js');
var Tabbable = require('../Tabbable/Tabbable.js');
var __chunk_6 = require('../chunk-d4f443f9.js');
var __chunk_7 = require('../chunk-ebc3ed80.js');
var CheckboxState = require('./CheckboxState.js');

var defaultClickKeys = [" "];
var useCheckbox = createHook.unstable_createHook({
  name: "Checkbox",
  compose: Tabbable.useTabbable,
  useState: CheckboxState.useCheckboxState,
  keys: ["value", "checked"],
  useProps: function useProps(options, _ref) {
    var htmlOnChange = _ref.onChange,
        htmlProps = __chunk_3._objectWithoutPropertiesLoose(_ref, ["onChange"]);

    var ref = React.useRef(null);
    var isBoolean = typeof options.value === "undefined";
    var checked = typeof options.checked !== "undefined" ? options.checked : isBoolean ? Boolean(options.state) : (options.state || []).indexOf(options.value) !== -1;
    React.useEffect(function () {
      if (!ref.current) {
        process.env.NODE_ENV !== "production" ? __chunk_7.warning(options.state === "indeterminate", "Can't set indeterminate state because either `ref` wasn't passed to component or the component wasn't rendered. See https://reakit.io/docs/checkbox", "Checkbox") : void 0;
        return;
      }

      if (options.state === "indeterminate") {
        ref.current.indeterminate = true;
      } else if (ref.current.indeterminate) {
        ref.current.indeterminate = false;
      }
    }, [options.state]);
    var onChange = React.useCallback(function (event) {
      if (options.disabled) return;

      if (htmlOnChange) {
        htmlOnChange(event);
      }

      if (!options.setState) return;

      if (isBoolean) {
        options.setState(!checked);
      } else {
        var array = Array.isArray(options.state) ? options.state : [];
        var index = array.indexOf(options.value);

        if (index === -1) {
          options.setState([].concat(array, [options.value]));
        } else {
          options.setState(__chunk_6.removeIndexFromArray(array, index));
        }
      }
    }, [htmlOnChange, options.disabled, options.setState, options.state, options.value]);
    var onClick = React.useCallback(function (event) {
      if (event.target instanceof HTMLInputElement) return;
      onChange(event);
    }, [onChange]);
    return mergeProps.unstable_mergeProps({
      ref: ref,
      checked: checked,
      "aria-checked": options.state === "indeterminate" ? "mixed" : checked,
      value: options.value,
      role: "checkbox",
      type: "checkbox",
      onChange: onChange,
      onClick: onClick
    }, htmlProps);
  },
  useCompose: function useCompose(options, htmlProps) {
    return Tabbable.useTabbable(__chunk_3._objectSpread({}, options, {
      unstable_clickKeys: defaultClickKeys
    }), htmlProps);
  }
});
var Checkbox = createComponent.unstable_createComponent({
  as: "input",
  useHook: useCheckbox
});

exports.Checkbox = Checkbox;
exports.useCheckbox = useCheckbox;
