'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
require('../utils/useSealedState.js');
require('../utils/useId.js');
var useCreateElement = require('../utils/useCreateElement.js');
require('../system/SystemContext.js');
require('../chunk-8e8e1bad.js');
require('../chunk-2138b837.js');
var mergeProps = require('../utils/mergeProps.js');
var __chunk_3 = require('../chunk-37dea005.js');
require('../system/useToken.js');
require('../system/useOptions.js');
require('../system/useProps.js');
require('../chunk-c06eec31.js');
var createHook = require('../utils/createHook.js');
var createComponent = require('../utils/createComponent.js');
require('../Box/Box.js');
require('../chunk-9263130a.js');
require('../Tabbable/Tabbable.js');
require('../chunk-d4f443f9.js');
var __chunk_7 = require('../chunk-ebc3ed80.js');
require('react-dom');
require('../Portal/Portal.js');
require('../Hidden/HiddenState.js');
require('../Hidden/Hidden.js');
require('../Dialog/Dialog.js');
require('body-scroll-lock');
require('../chunk-5a603fe6.js');
require('../Dialog/DialogState.js');
var __chunk_14 = require('../chunk-498e747a.js');
require('../Rover/RoverState.js');
require('../Rover/Rover.js');
require('popper.js');
require('../Popover/PopoverState.js');
var Popover = require('../Popover/Popover.js');
var StaticMenu = require('./StaticMenu.js');
var __chunk_15 = require('../chunk-8a7a2ea8.js');
var MenuState = require('./MenuState.js');

var useMenu = createHook.unstable_createHook({
  name: "Menu",
  compose: [StaticMenu.useStaticMenu, Popover.usePopover],
  useState: MenuState.useMenuState,
  propsAreEqual: function propsAreEqual(prev, next) {
    if (prev.visible === false && next.visible === false) {
      return true;
    }

    return null;
  },
  useOptions: function useOptions(options) {
    var parent = React.useContext(__chunk_15.MenuContext);
    var parentIsHorizontal = parent && parent.orientation === "horizontal";
    return __chunk_3._objectSpread({
      unstable_autoFocusOnShow: !parent,
      unstable_autoFocusOnHide: !parentIsHorizontal
    }, options);
  },
  useProps: function useProps(options, htmlProps) {
    var parent = React.useContext(__chunk_15.MenuContext);
    var ref = React.useRef(null);
    var isHorizontal = options.orientation === "horizontal";
    var isVertical = options.orientation === "vertical";
    var horizontalParent = parent;

    while (horizontalParent && horizontalParent.orientation !== "horizontal") {
      horizontalParent = horizontalParent.parent;
    }

    var _options$placement$sp = options.placement.split("-"),
        dir = _options$placement$sp[0];

    var rovingBindings = React.useMemo(function () {
      return __chunk_14.createOnKeyDown({
        stopPropagation: function stopPropagation(event) {
          // On Esc, only stop propagation if there's no parent menu
          // Otherwise, pressing Esc should close all menus
          if (event.key === "Escape" && parent) return false;
          return true;
        },
        keyMap: function keyMap(event) {
          process.env.NODE_ENV !== "production" ? __chunk_7.warning(!ref.current, "Can't detect arrow keys because `ref` wasn't passed to component. See https://reakit.io/docs/menu", "Menu") : void 0;
          var targetIsMenu = event.target === ref.current;
          return {
            Escape: options.hide,
            ArrowUp: targetIsMenu && !isHorizontal && options.last,
            ArrowRight: targetIsMenu && !isVertical && options.first,
            ArrowDown: targetIsMenu && !isHorizontal && options.first,
            ArrowLeft: targetIsMenu && !isVertical && options.last,
            Home: targetIsMenu && options.first,
            End: targetIsMenu && options.last,
            PageUp: targetIsMenu && options.first,
            PageDown: targetIsMenu && options.last
          };
        }
      });
    }, [parent, options.hide, options.last, options.first]);
    var parentBindings = React.useMemo(function () {
      return __chunk_14.createOnKeyDown({
        stopPropagation: true,
        shouldKeyDown: function shouldKeyDown(event) {
          return Boolean(parent && ref.current && ref.current.contains(event.target));
        },
        keyMap: parent ? {
          ArrowRight: horizontalParent && dir !== "left" ? horizontalParent.next : dir === "left" && options.hide,
          ArrowLeft: horizontalParent && dir !== "right" ? horizontalParent.previous : dir === "right" && options.hide
        } : {}
      });
    }, [Boolean(parent), horizontalParent && horizontalParent.next, horizontalParent && horizontalParent.previous, dir, options.hide]);
    var onKeyDown = React.useCallback(function (event) {
      rovingBindings(event);
      parentBindings(event);
    }, [rovingBindings, parentBindings]);
    return mergeProps.unstable_mergeProps({
      ref: ref,
      role: "menu",
      onKeyDown: onKeyDown
    }, htmlProps);
  },
  useCompose: function useCompose(options, htmlProps) {
    htmlProps = StaticMenu.useStaticMenu(options, htmlProps);
    return Popover.usePopover(__chunk_3._objectSpread({}, options, {
      modal: false,
      unstable_portal: false,
      unstable_orphan: false,
      hideOnEsc: false
    }), htmlProps);
  }
});
var Menu = createComponent.unstable_createComponent({
  as: "div",
  useHook: useMenu,
  useCreateElement: function useCreateElement$1(type, props, children) {
    process.env.NODE_ENV !== "production" ? __chunk_7.warning(!props["aria-label"] && !props["aria-labelledby"], "You should provide either `aria-label` or `aria-labelledby` props. See https://reakit.io/docs/menu", "Menu") : void 0;
    return useCreateElement.unstable_useCreateElement(type, props, children);
  }
});

exports.Menu = Menu;
exports.useMenu = useMenu;
