'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
require('../utils/useSealedState.js');
require('../utils/useId.js');
require('../utils/useCreateElement.js');
require('../system/SystemContext.js');
require('../chunk-8e8e1bad.js');
require('../chunk-2138b837.js');
var mergeProps = require('../utils/mergeProps.js');
require('../chunk-37dea005.js');
require('../system/useToken.js');
require('../system/useOptions.js');
require('../system/useProps.js');
require('../chunk-c06eec31.js');
var createHook = require('../utils/createHook.js');
var createComponent = require('../utils/createComponent.js');
require('../Box/Box.js');
require('../chunk-9263130a.js');
require('../Tabbable/Tabbable.js');
var __chunk_7 = require('../chunk-ebc3ed80.js');
require('../Hidden/HiddenState.js');
require('../chunk-5a603fe6.js');
require('../Dialog/DialogState.js');
require('../chunk-498e747a.js');
require('../Rover/RoverState.js');
var Rover = require('../Rover/Rover.js');
require('popper.js');
require('../Popover/PopoverState.js');
require('../chunk-8a7a2ea8.js');
var MenuState = require('./MenuState.js');

// TODO: Find a better implementation
function isTouchDevice() {
  if (process.env.NODE_ENV === "test" || typeof window === "undefined") return false;
  return "ontouchstart" in window || navigator.maxTouchPoints > 0 || navigator.msMaxTouchPoints > 0;
}

var useMenuItem = createHook.unstable_createHook({
  name: "MenuItem",
  compose: Rover.useRover,
  useState: MenuState.useMenuState,
  useProps: function useProps(options, htmlProps) {
    var ref = React.useRef(null);
    var onMouseOver = React.useCallback(function () {
      if (options.orientation !== "horizontal" && !isTouchDevice()) {
        if (!ref.current) {
          process.env.NODE_ENV !== "production" ? __chunk_7.warning(true, "Can't respond to mouse over on `MenuItem` because `ref` wasn't passed to component. See https://reakit.io/docs/menu", "MenuItem") : void 0;
          return;
        }

        ref.current.focus();
      }
    }, [options.orientation]);
    var onMouseOut = React.useCallback(function () {
      if (ref.current) {
        // Ignores disclosure
        if (!ref.current.hasAttribute("aria-controls") || ref.current.getAttribute("aria-expanded") !== "true") {
          ref.current.blur();
        }

        var menu = ref.current.closest("[role=menu],[role=menubar]");

        if (menu) {
          var nestedMenu = menu.querySelector("[role=menu]:not([hidden]),[role=menubar]:not([hidden])");

          if (!nestedMenu && !isTouchDevice()) {
            options.move(null);
            menu.focus();
          }
        }
      }
    }, [options.move]);
    return mergeProps.unstable_mergeProps({
      ref: ref,
      role: "menuitem",
      onMouseOver: onMouseOver,
      onMouseOut: onMouseOut
    }, htmlProps);
  }
});
var MenuItem = createComponent.unstable_createComponent({
  as: "button",
  useHook: useMenuItem
});

exports.MenuItem = MenuItem;
exports.useMenuItem = useMenuItem;
