'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
require('../utils/useCreateElement.js');
require('../system/SystemContext.js');
require('../chunk-8e8e1bad.js');
require('../chunk-2138b837.js');
var mergeProps = require('../utils/mergeProps.js');
var __chunk_3 = require('../chunk-37dea005.js');
require('../system/useToken.js');
require('../system/useOptions.js');
require('../system/useProps.js');
require('../chunk-c06eec31.js');
var createHook = require('../utils/createHook.js');
var createComponent = require('../utils/createComponent.js');
var Box = require('../Box/Box.js');
var __chunk_5 = require('../chunk-9263130a.js');

function isNativeTabbable(element) {
  if (element instanceof HTMLButtonElement) return true;
  if (element instanceof HTMLInputElement) return true;
  if (element instanceof HTMLSelectElement) return true;
  if (element instanceof HTMLTextAreaElement) return true;
  if (element instanceof HTMLAnchorElement) return true;
  if (element instanceof HTMLAudioElement) return true;
  if (element instanceof HTMLVideoElement) return true;
  return false;
}

var defaultClickKeys = ["Enter", " "];
var useTabbable = createHook.unstable_createHook({
  name: "Tabbable",
  compose: Box.useBox,
  keys: ["disabled", "focusable", "unstable_clickKeys"],
  useOptions: function useOptions(_ref, htmlProps) {
    var _ref$unstable_clickKe = _ref.unstable_clickKeys,
        unstable_clickKeys = _ref$unstable_clickKe === void 0 ? defaultClickKeys : _ref$unstable_clickKe,
        options = __chunk_3._objectWithoutPropertiesLoose(_ref, ["unstable_clickKeys"]);

    return __chunk_3._objectSpread({
      unstable_clickKeys: unstable_clickKeys,
      disabled: htmlProps.disabled
    }, options);
  },
  useProps: function useProps(options, _ref2) {
    var _ref2$tabIndex = _ref2.tabIndex,
        tabIndex = _ref2$tabIndex === void 0 ? 0 : _ref2$tabIndex,
        onClick = _ref2.onClick,
        onMouseOver = _ref2.onMouseOver,
        onMouseDown = _ref2.onMouseDown,
        htmlProps = __chunk_3._objectWithoutPropertiesLoose(_ref2, ["tabIndex", "onClick", "onMouseOver", "onMouseDown"]);

    var ref = React.useRef(null);
    var clickKeysRef = __chunk_5.useLiveRef(options.unstable_clickKeys);
    var trulyDisabled = options.disabled && !options.focusable;
    return mergeProps.unstable_mergeProps({
      ref: ref,
      disabled: trulyDisabled,
      tabIndex: trulyDisabled ? undefined : tabIndex,
      "aria-disabled": options.disabled,
      onMouseDown: React.useCallback(function (event) {
        if (event.target instanceof HTMLInputElement) {
          if (onMouseDown) {
            onMouseDown(event);
          }

          return;
        }

        event.preventDefault();

        if (options.disabled) {
          event.stopPropagation();
        } else {
          (ref.current || event.target).focus();

          if (onMouseDown) {
            onMouseDown(event);
          }
        }
      }, [options.disabled, onMouseDown]),
      onClick: React.useCallback(function (event) {
        if (options.disabled) {
          event.stopPropagation();
          event.preventDefault();
        } else if (onClick) {
          onClick(event);
        }
      }, [options.disabled, onClick]),
      onMouseOver: React.useCallback(function (event) {
        if (options.disabled) {
          event.stopPropagation();
          event.preventDefault();
        } else if (onMouseOver) {
          onMouseOver(event);
        }
      }, [options.disabled, onMouseOver]),
      onKeyDown: React.useCallback(function (event) {
        if (options.disabled) return;

        if (clickKeysRef.current === defaultClickKeys && isNativeTabbable(event.target)) {
          return;
        }

        if (clickKeysRef.current && clickKeysRef.current.indexOf(event.key) !== -1) {
          event.preventDefault();
          event.target.dispatchEvent(new MouseEvent("click", {
            view: window,
            bubbles: true,
            cancelable: false
          }));
        }
      }, [options.disabled])
    }, htmlProps);
  }
});
var Tabbable = createComponent.unstable_createComponent({
  as: "button",
  useHook: useTabbable
});

exports.Tabbable = Tabbable;
exports.useTabbable = useTabbable;
